@@ -21,6 +21,7 @@
 */
 
 #include "pool.h"
+#include <unistd.h>
 #include "context/pool_session_context.h"
 #include "protocol/pool_process_query.h"
 #include "protocol/pool_proto_modules.h"
@@ -34,11 +35,11 @@
 #include "utils/palloc.h"
 #include "utils/memutils.h"
 #include "auth/md5.h"
+#include <unistd.h>
 
 #ifdef HAVE_CRYPT_H
 #include <crypt.h>
 #endif
-
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
 #endif
@@ -1449,6 +1450,9 @@ authenticate_frontend_SCRAM(POOL_CONNECTION * backend, POOL_CONNECTION * fronten
 	frontend->frontend_authenticated = true;
 }
 
+/*
+ * Authenticate frontend using pool_hba.conf
+ */
 void
 authenticate_frontend(POOL_CONNECTION * frontend)
 {
