@@ -4,7 +4,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2023	PgPool Global Development Group
+ * Copyright (c) 2003-2024	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -47,19 +47,6 @@
 #define POOLKEYFILE 	".pgpoolkey"
 #define POOLKEYFILEENV "PGPOOLKEYFILE"
 
-/*
- * Brought from PostgreSQL's pg_config_manual.h.
- *
- * Maximum length for identifiers (e.g. table names, column names,
- * function names).  Names actually are limited to one less byte than this,
- * because the length must include a trailing zero byte.
- *
- * Please note that in version 2 protocol, maximum user name length is
- * SM_USER, which is 32.
- */
-#define NAMEDATALEN 64
-
-/* configuration file name */
 #define POOL_CONF_FILE_NAME "pgpool.conf"
 
 /* PCP user/password file name */
@@ -83,6 +70,9 @@
 /* status file name */
 #define STATUS_FILE_NAME "pgpool_status"
 
+/* query cache lock file name */
+#define QUERY_CACHE_LOCK_FILE "memq_lock_file"
+
 /* default string used to identify pgpool on syslog output */
 #define DEFAULT_SYSLOG_IDENT "pgpool"
 
@@ -276,7 +266,12 @@ typedef struct
 
 typedef struct
 {
-	ConnectionInfo *info;		/* connection info on shmem */
+	/*
+	 * info is initialized by pool_init_cp() at the startup of pgpool child
+	 * process. pool_coninfo(child id, pool index, backend id = 0) is used to
+	 * refer to the appropriate pointer on shared memory.
+	 */
+	ConnectionInfo *info;
 	POOL_CONNECTION_POOL_SLOT *slots[MAX_NUM_BACKENDS];
 }			POOL_CONNECTION_POOL;
 
@@ -353,9 +348,9 @@ extern int	my_main_node_id;
 typedef enum
 {
 	POOL_NODE_STATUS_UNUSED,	/* unused */
-	POOL_NODE_STATUS_PRIMARY,	/* primary ndoe */
+	POOL_NODE_STATUS_PRIMARY,	/* primary node */
 	POOL_NODE_STATUS_STANDBY,	/* standby node */
-	POOL_NODE_STATUS_INVALID	/* invalid node (split branin, stand alone) */
+	POOL_NODE_STATUS_INVALID	/* invalid node (split brain, stand alone) */
 }			POOL_NODE_STATUS;
 
 /* Clustering mode macros */
@@ -378,16 +373,15 @@ typedef enum
 #define Min(x, y)		((x) < (y) ? (x) : (y))
 
 
-#define MAX_NUM_SEMAPHORES		9
+#define MAX_NUM_SEMAPHORES		8
 #define CONN_COUNTER_SEM		0
 #define REQUEST_INFO_SEM		1
-#define SHM_CACHE_SEM			2
-#define QUERY_CACHE_STATS_SEM	3
-#define PCP_REQUEST_SEM			4
-#define ACCEPT_FD_SEM			5
-#define SI_CRITICAL_REGION_SEM	6
-#define FOLLOW_PRIMARY_SEM		7
-#define MAIN_EXIT_HANDLER_SEM	8	/* used in exit_hander in pgpool main process */
+#define QUERY_CACHE_STATS_SEM	2
+#define PCP_REQUEST_SEM			3
+#define ACCEPT_FD_SEM			4
+#define SI_CRITICAL_REGION_SEM	5
+#define FOLLOW_PRIMARY_SEM		6
+#define MAIN_EXIT_HANDLER_SEM	7	/* used in exit_hander in pgpool main process */
 #define MAX_REQUEST_QUEUE_SIZE	10
 
 #define MAX_SEC_WAIT_FOR_CLUSTER_TRANSACTION 10	/* time in seconds to keep
@@ -465,6 +459,7 @@ typedef struct
 	bool		follow_primary_lock_held_remotely; /* true when lock is held by
 													watchdog coordinator*/
 	bool		follow_primary_ongoing;	/* true if follow primary command is ongoing */
+	bool		query_cache_invalidate_request;	/* true if pcp_invalidate_query_cache requested */
 }			POOL_REQUEST_INFO;
 
 /* description of row. corresponding to RowDescription message */
@@ -507,7 +502,7 @@ typedef enum
 }			POOL_RECOVERY_MODE;
 
 /*
- * Process types.  DO NOT change the order of each enum meber!  If you do
+ * Process types.  DO NOT change the order of each enum member!  If you do
  * that, you must change application_name array in src/main/pgpool_main.c
  * accordingly.
  */
@@ -593,6 +588,7 @@ extern BACKEND_STATUS private_backend_status[MAX_NUM_BACKENDS];
 extern char remote_host[];		/* client host */
 extern char remote_port[];		/* client port */
 
+
 /*
  * public functions
  */
@@ -604,7 +600,7 @@ extern char *get_pool_key(void);
 
 
 /*pcp_child.c*/
-extern void pcp_main(int unix_fd, int inet_fd);
+extern void pcp_main(int *fds);
 
 
 
@@ -629,7 +625,7 @@ extern void initialize_shared_memory_main_segment(size_t size);
 extern void * pool_shared_memory_segment_get_chunk(size_t size);
 
 
-/* pool_main.c*/
+/* pgpool_main.c*/
 extern BackendInfo * pool_get_node_info(int node_number);
 extern int	pool_get_node_count(void);
 extern int *pool_get_process_list(int *array_size);
@@ -646,6 +642,7 @@ extern void pool_set_backend_status_changed_time(int backend_id);
 extern int	get_next_main_node(void);
 extern bool pool_acquire_follow_primary_lock(bool block, bool remote_reques);
 extern void pool_release_follow_primary_lock(bool remote_reques);
+extern void pool_signal_logrotate(void);
 
 /* strlcpy.c */
 #ifndef HAVE_STRLCPY
