@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2022	PgPool Global Development Group
+ * Copyright (c) 2003-2024	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -154,6 +154,14 @@ do_health_check_child(int *node_id)
 	/* Initialize per process context */
 	pool_init_process_context();
 
+	/*
+	 * Open pool_passwd.
+	 */
+	if (strcmp("", pool_config->pool_passwd))
+	{
+		pool_reopen_passwd_file();
+	}
+
 	if (sigsetjmp(local_sigjmp_buf, 1) != 0)
 	{
 		pool_signal(SIGALRM, SIG_IGN);
@@ -172,6 +180,12 @@ do_health_check_child(int *node_id)
 	{
 		MemoryContextSwitchTo(HealthCheckMemoryContext);
 		MemoryContextResetAndDeleteChildren(HealthCheckMemoryContext);
+		/*
+		 * Since HealthCheckMemoryContext is used for "slot", we need to clear it
+		 * so that new slot is allocated later on.
+		 */
+		slot = NULL;
+
 		bool	skipped = false;
 
 		CHECK_REQUEST;
@@ -309,13 +323,13 @@ establish_persistent_connection(int node)
 	/*
 	 * If the node is already in down status or unused, do nothing.
 	 * except when the node state is down because of quarantine operation
-	 * since we want to detect when the node cames back to life again to
+	 * since we want to detect when the node comes back to life again to
 	 * remove it from the quarantine state
 	 */
 	if (bkinfo->backend_status == CON_UNUSED ||
 		(bkinfo->backend_status == CON_DOWN && bkinfo->quarantine == false))
 	{
-		/* get current time to use auto_faliback_interval */
+		/* get current time to use auto_failback_interval */
 		now = time(NULL);
 
 		if (pool_config->auto_failback && auto_failback_interval < now &&
@@ -434,7 +448,7 @@ establish_persistent_connection(int node)
 		{
 				ereport(LOG,
 					(errmsg("request auto failback, node id:%d", node)));
-				/* get current time to use auto_faliback_interval */
+				/* get current time to use auto_failback_interval */
 				now = time(NULL);
 				auto_failback_interval = now + pool_config->auto_failback_interval;
 
