@@ -3,7 +3,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2021	PgPool Global Development Group
+ * Copyright (c) 2003-2024	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -162,7 +162,14 @@ do_worker_child(void)
 	{
 		MemoryContextSwitchTo(WorkerMemoryContext);
 		MemoryContextResetAndDeleteChildren(WorkerMemoryContext);
-		WD_STATES	wd_status;
+		/*
+		 * Since WorkerMemoryContext is used for "slots", we need to clear it
+		 * so that new slots are allocated later on.
+		 */
+		memset(slots, 0, sizeof(slots));
+		
+		bool	watchdog_leader;	/* true if I am the watchdog leader */
+
 
 		CHECK_REQUEST;
 
@@ -174,11 +181,48 @@ do_worker_child(void)
 		/*
 		 * Get watchdog status if watchdog is enabled.
 		 */
+		watchdog_leader = false;
 		if (pool_config->use_watchdog)
 		{
+			WD_STATES	wd_status;
+			WDPGBackendStatus	*backendStatus;
+
 			wd_status = wd_internal_get_watchdog_local_node_state();
 			ereport(DEBUG1,
 					(errmsg("watchdog status: %d", wd_status)));
+			/*
+			 * Ask the watchdog to get all the backend states from the
+			 * Leader/Coordinator Pgpool-II node.
+			 */
+			watchdog_leader = false;
+			backendStatus = get_pg_backend_status_from_leader_wd_node();
+
+			if (!backendStatus)
+				/*
+				 * Couldn't get leader status.
+				 */
+				watchdog_leader = false;
+			else
+			{
+				int	quorum = wd_internal_get_watchdog_quorum_state();
+				int	node_count = backendStatus->node_count;
+
+				ereport(DEBUG1,
+						(errmsg("quorum: %d node_count: %d",
+								quorum, node_count)));
+				if (quorum >= 0 && backendStatus->node_count <= 0)
+				{
+					/*
+					 * Quorum exists and node_count <= 0.
+					 * Definitely I am the leader.
+					 */
+					watchdog_leader = true;
+				}
+				else
+					watchdog_leader = false;
+
+				pfree(backendStatus);
+			}
 		}
 
 		/*
@@ -189,7 +233,7 @@ do_worker_child(void)
 			Req_info->switching == false)
 		{
 			/*
-			 * Acquire follow primary lock. If fail to acqure lock, try again.
+			 * Acquire follow primary lock. If fail to acquire lock, try again.
 			 */
 			follow_primary_lock_acquired = false;
 
@@ -227,8 +271,33 @@ do_worker_child(void)
 							 */
 							if (pool_config->detach_false_primary)
 							{
-								n = i;
-								degenerate_backend_set(&n, 1, REQ_DETAIL_SWITCHOVER);
+								/*
+								 * However if watchdog is enabled and I am not
+								 * the leader, do not detach the invalid node
+								 * because the information to determine the
+								 * false primary might be outdated or
+								 * temporarily inconsistent.  See
+								 * [pgpool-hackers: 4431] for more details.
+								 */
+								if (!pool_config->use_watchdog ||
+									(pool_config->use_watchdog && watchdog_leader))
+								{
+									n = i;
+									/*
+									 * In the case watchdog enabled, we need
+									 * to add REQ_DETAIL_CONFIRMED, which
+									 * means no quorum consensus is
+									 * required. If we do not add this, the
+									 * target node will remain quarantine
+									 * state since other node does not request
+									 * failover.
+									 */
+									degenerate_backend_set(&n, 1,
+														   REQ_DETAIL_SWITCHOVER|REQ_DETAIL_CONFIRMED);
+								}
+								else if (pool_config->use_watchdog)
+									ereport(LOG,
+											(errmsg("do not detach invalid node %d because I am not the leader or quorum does not exist", i)));
 							}
 						}
 					}
@@ -319,13 +388,15 @@ check_replication_time_lag(void)
 	int			i;
 	POOL_SELECT_RESULT *res;
 	POOL_SELECT_RESULT *res_rep;	/* query results of pg_stat_replication */
-	unsigned long long int lsn[MAX_NUM_BACKENDS];
+	uint64	lsn[MAX_NUM_BACKENDS];
 	char	   *query;
 	char	   *stat_rep_query;
 	BackendInfo *bkinfo;
-	unsigned long long int lag;
+	uint64	lag;
+	uint64	delay_threshold_by_time;
 	ErrorContextCallback callback;
 	int		active_standby_node;
+	bool	replication_delay_by_time;
 
 	/* clear replication state */
 	for (i = 0; i < NUM_BACKENDS; i++)
@@ -371,6 +442,7 @@ check_replication_time_lag(void)
 	error_context_stack = &callback;
 	stat_rep_query = NULL;
 	active_standby_node = 0;
+	replication_delay_by_time = false;
 
 	for (i = 0; i < NUM_BACKENDS; i++)
 	{
@@ -413,9 +485,15 @@ check_replication_time_lag(void)
 				query = "SELECT pg_catalog.pg_current_xlog_location()";
 
 			if (server_version[i] == PG91_SERVER_VERSION)
-				stat_rep_query = "SELECT application_name, state, '' AS sync_state FROM pg_catalog.pg_stat_replication";
+				stat_rep_query = "SELECT application_name, state, '' AS sync_state, '' AS replay_lag FROM pg_catalog.pg_stat_replication";
+			else if (server_version[i] >= PG10_SERVER_VERSION)
+			{
+				stat_rep_query = "SELECT application_name, state, sync_state,(EXTRACT(EPOCH FROM replay_lag)*1000000)::BIGINT FROM pg_catalog.pg_stat_replication";
+				if (pool_config->delay_threshold_by_time > 0)
+					replication_delay_by_time = true;
+			}
 			else if (server_version[i] > PG91_SERVER_VERSION)
-				stat_rep_query = "SELECT application_name, state, sync_state FROM pg_catalog.pg_stat_replication";
+				stat_rep_query = "SELECT application_name, state, sync_state, '' AS replay_lag FROM pg_catalog.pg_stat_replication";
 		}
 		else
 		{
@@ -463,10 +541,11 @@ check_replication_time_lag(void)
 			{
 				int		j;
 				char	*s;
+#define	NUM_COLS 4
 
 				for (j = 0; j < res_rep->numrows; j++)
 				{
-					if (strcmp(res_rep->data[j*3], bkinfo->backend_application_name) == 0)
+					if (strcmp(res_rep->data[j*NUM_COLS], bkinfo->backend_application_name) == 0)
 					{
 						/*
 						 * If sr_check_user has enough privilege, it should return
@@ -474,10 +553,21 @@ check_replication_time_lag(void)
 						 * res_rep->data[1] and [2]. So we need to prepare for the
 						 * latter case.
 						 */
-						s = res_rep->data[j*3+1]? res_rep->data[j*3+1] : "";
+						s = res_rep->data[j*NUM_COLS+1]? res_rep->data[j*NUM_COLS+1] : "";
 						strlcpy(bkinfo->replication_state, s, NAMEDATALEN);
-						s = res_rep->data[j*3+2]? res_rep->data[j*3+2] : "";
+
+						s = res_rep->data[j*NUM_COLS+2]? res_rep->data[j*NUM_COLS+2] : "";
 						strlcpy(bkinfo->replication_sync_state, s, NAMEDATALEN);
+
+						s = res_rep->data[j*NUM_COLS+3];
+						if (s)
+						{
+							bkinfo->standby_delay = atol(s);
+							ereport(DEBUG1,
+									(errmsg("standby delay in milli seconds * 1000: " UINT64_FORMAT "", bkinfo->standby_delay)));
+						}
+						else
+							bkinfo->standby_delay = 0;
 					}
 				}
 			}
@@ -501,17 +591,52 @@ check_replication_time_lag(void)
 		}
 		else
 		{
-			bkinfo->standby_delay = lag;
+			if (replication_delay_by_time)
+			{
+				/*
+				 * If replication delay is measured by time, indicate it in
+				 * shared memory area.
+				 */
+				bkinfo->standby_delay_by_time = true;
+			}
+			else
+			{
+				/*
+				 * If replication delay is not measured by time, set the LSN
+				 * lag to shared memory area.
+				 */
+				bkinfo->standby_delay = lag;
+				bkinfo->standby_delay_by_time = false;
+			}
 
 			/* Log delay if necessary */
-			if ((pool_config->log_standby_delay == LSD_ALWAYS && lag > 0) ||
-				(pool_config->delay_threshold &&
-				 pool_config->log_standby_delay == LSD_OVER_THRESHOLD &&
-				 lag > pool_config->delay_threshold))
+			if (replication_delay_by_time)
 			{
-				ereport(LOG,
-						(errmsg("Replication of node:%d is behind %llu bytes from the primary server (node:%d)",
-								i, lsn[PRIMARY_NODE_ID] - lsn[i], PRIMARY_NODE_ID)));
+				lag = bkinfo->standby_delay;
+				delay_threshold_by_time = pool_config->delay_threshold_by_time;
+				delay_threshold_by_time *= 1000;	/* convert from milli seconds to micro seconds */
+
+				/* Log delay if necessary */
+				if ((pool_config->log_standby_delay == LSD_ALWAYS && lag > 0) ||
+					(pool_config->log_standby_delay == LSD_OVER_THRESHOLD &&
+					 lag > delay_threshold_by_time))
+				{
+					ereport(LOG,
+							(errmsg("Replication of node: %d is behind %.6f second(s) from the primary server (node: %d)",
+									i, ((float)lag)/1000000, PRIMARY_NODE_ID)));
+				}
+			}
+			else
+			{
+				if ((pool_config->log_standby_delay == LSD_ALWAYS && lag > 0) ||
+					(pool_config->delay_threshold &&
+					 pool_config->log_standby_delay == LSD_OVER_THRESHOLD &&
+					 lag > pool_config->delay_threshold))
+				{
+					ereport(LOG,
+							(errmsg("Replication of node: %d is behind " UINT64_FORMAT " bytes from the primary server (node: %d)",
+									i, (uint64)(lsn[PRIMARY_NODE_ID] - lsn[i]), PRIMARY_NODE_ID)));
+				}
 			}
 		}
 	}
