@@ -6,7 +6,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2016	PgPool Global Development Group
+ * Copyright (c) 2003-2022	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -39,11 +39,13 @@
 #include "utils/elog.h"
 #include "utils/palloc.h"
 #include "utils/memutils.h"
+#include "utils/pool_signal.h"
+#include "utils/ps_status.h"
 
 #include "watchdog/wd_utils.h"
 #include "watchdog/wd_lifecheck.h"
 #include "watchdog/wd_ipc_defines.h"
-#include "watchdog/wd_ipc_commands.h"
+#include "watchdog/wd_internal_commands.h"
 #include "watchdog/wd_json_data.h"
 
 #include "libpq-fe.h"
@@ -70,7 +72,6 @@ typedef struct WdUpstreamConnectionData
 	char	   *hostname;		/* host name of server */
 	pid_t		pid;			/* pid of ping process */
 	bool		reachable;		/* true if last ping was successful */
-	int			outputfd;		/* pipe fd linked to output of ping process */
 }			WdUpstreamConnectionData;
 
 
@@ -132,7 +133,7 @@ lifecheck_child_name(pid_t pid)
 {
 	int			i;
 
-	for (i = 0; i < pool_config->num_hb_if; i++)
+	for (i = 0; i < pool_config->num_hb_dest_if; i++)
 	{
 		if (g_hb_receiver_pid && pid == g_hb_receiver_pid[i])
 			return "heartBeat receiver";
@@ -166,9 +167,8 @@ reaper(void)
 
 		if (server)
 		{
-			server->reachable = wd_get_ping_result(server->hostname, status, server->outputfd);
+			server->reachable = (status == 0);
 			server->pid = 0;
-			close(server->outputfd);
 		}
 		else
 			wd_reaper_lifecheck(pid, status);
@@ -211,13 +211,13 @@ wd_reaper_lifecheck(pid_t pid, int status)
 	if (g_hb_receiver_pid == NULL && g_hb_sender_pid == NULL)
 		return -1;
 
-	for (i = 0; i < pool_config->num_hb_if; i++)
+	for (i = 0; i < pool_config->num_hb_dest_if; i++)
 	{
 		if (g_hb_receiver_pid && pid == g_hb_receiver_pid[i])
 		{
 			if (restart_child)
 			{
-				g_hb_receiver_pid[i] = wd_hb_receiver(1, &(pool_config->hb_if[i]));
+				g_hb_receiver_pid[i] = wd_hb_receiver(1, &(pool_config->hb_dest_if[i]));
 				ereport(LOG,
 						(errmsg("fork a new %s process with pid: %d", proc_name, g_hb_receiver_pid[i])));
 			}
@@ -231,7 +231,7 @@ wd_reaper_lifecheck(pid_t pid, int status)
 		{
 			if (restart_child)
 			{
-				g_hb_sender_pid[i] = wd_hb_sender(1, &(pool_config->hb_if[i]));
+				g_hb_sender_pid[i] = wd_hb_sender(1, &(pool_config->hb_dest_if[i]));
 				ereport(LOG,
 						(errmsg("fork a new %s process with pid: %d", proc_name, g_hb_sender_pid[i])));
 			}
@@ -254,7 +254,7 @@ lifecheck_kill_all_children(int sig)
 	{
 		int			i;
 
-		for (i = 0; i < pool_config->num_hb_if; i++)
+		for (i = 0; i < pool_config->num_hb_dest_if; i++)
 		{
 			pid_t		pid_child = g_hb_receiver_pid[i];
 
@@ -315,7 +315,8 @@ lifecheck_exit_handler(int sig)
 
 		if (wpid == -1 && errno != ECHILD)
 			ereport(WARNING,
-					(errmsg("wait() on lifecheck children failed. reason:%s", strerror(errno))));
+					(errmsg("wait() on lifecheck children failed"),
+					 errdetail("%m")));
 
 		if (g_hb_receiver_pid)
 			pfree(g_hb_receiver_pid);
@@ -354,9 +355,7 @@ fork_lifecheck_child(void)
 	if (pid == 0)
 	{
 		on_exit_reset();
-
-		/* Set the process type variable */
-		processType = PT_LIFECHECK;
+		SetProcessGlobalVariables(PT_LIFECHECK);
 
 		/* call lifecheck child main */
 		POOL_SETMASK(&UnBlockSig);
@@ -365,7 +364,8 @@ fork_lifecheck_child(void)
 	else if (pid == -1)
 	{
 		ereport(FATAL,
-				(errmsg("fork() failed. reason: %s", strerror(errno))));
+				(errmsg("fork() failed"),
+				 errdetail("%m")));
 	}
 
 	return pid;
@@ -468,16 +468,16 @@ spawn_lifecheck_children(void)
 	{
 		int			i;
 
-		g_hb_receiver_pid = palloc0(sizeof(pid_t) * pool_config->num_hb_if);
-		g_hb_sender_pid = palloc0(sizeof(pid_t) * pool_config->num_hb_if);
+		g_hb_receiver_pid = palloc0(sizeof(pid_t) * pool_config->num_hb_dest_if);
+		g_hb_sender_pid = palloc0(sizeof(pid_t) * pool_config->num_hb_dest_if);
 
-		for (i = 0; i < pool_config->num_hb_if; i++)
+		for (i = 0; i < pool_config->num_hb_dest_if; i++)
 		{
 			/* heartbeat receiver process */
-			g_hb_receiver_pid[i] = wd_hb_receiver(1, &(pool_config->hb_if[i]));
+			g_hb_receiver_pid[i] = wd_hb_receiver(1, &(pool_config->hb_dest_if[i]));
 
 			/* heartbeat sender process */
-			g_hb_sender_pid[i] = wd_hb_sender(1, &(pool_config->hb_if[i]));
+			g_hb_sender_pid[i] = wd_hb_sender(1, &(pool_config->hb_dest_if[i]));
 		}
 	}
 }
@@ -532,7 +532,7 @@ inform_node_status(LifeCheckNode * node, char *message)
 	if (json_data == NULL)
 		return false;
 
-	for (x = 0; x < MAX_SEC_WAIT_FOR_CLUSTER_TRANSATION; x++)
+	for (x = 0; x < MAX_SEC_WAIT_FOR_CLUSTER_TRANSACTION; x++)
 	{
 		res = issue_command_to_watchdog(WD_NODE_STATUS_CHANGE_COMMAND, 0, json_data, strlen(json_data), false);
 		if (res)
@@ -551,7 +551,7 @@ inform_node_status(LifeCheckNode * node, char *message)
 static bool
 fetch_watchdog_nodes_data(void)
 {
-	char	   *json_data = wd_get_watchdog_nodes(-1);
+	char	   *json_data = wd_internal_get_watchdog_nodes_json(-1);
 
 	if (json_data == NULL)
 	{
@@ -567,6 +567,7 @@ fetch_watchdog_nodes_data(void)
 static void
 load_watchdog_nodes_from_json(char *json_data, int len)
 {
+	size_t shmem_size;
 	json_value *root;
 	json_value *value;
 	int			i,
@@ -615,14 +616,19 @@ load_watchdog_nodes_from_json(char *json_data, int len)
 	}
 
 	/* okay we are done, put this in shared memory */
-	gslifeCheckCluster = pool_shared_memory_create(sizeof(LifeCheckCluster));
+	shmem_size = MAXALIGN(sizeof(LifeCheckCluster));
+	shmem_size += MAXALIGN(sizeof(LifeCheckNode) * nodeCount);
+
+	gslifeCheckCluster = pool_shared_memory_create(shmem_size);
 	gslifeCheckCluster->nodeCount = nodeCount;
-	gslifeCheckCluster->lifeCheckNodes = pool_shared_memory_create(sizeof(LifeCheckNode) * gslifeCheckCluster->nodeCount);
+	gslifeCheckCluster->lifeCheckNodes = (LifeCheckNode*)((char*)gslifeCheckCluster + MAXALIGN(sizeof(LifeCheckCluster)));
 	for (i = 0; i < nodeCount; i++)
 	{
-		WDNodeInfo *nodeInfo = get_WDNodeInfo_from_wd_node_json(value->u.array.values[i]);
-
-		gslifeCheckCluster->lifeCheckNodes[i].nodeState = NODE_EMPTY;
+		WDNodeInfo *nodeInfo = parse_watchdog_node_info_from_wd_node_json(value->u.array.values[i]);
+		
+		gslifeCheckCluster->lifeCheckNodes[i].wdState = nodeInfo->state;
+		strcpy(gslifeCheckCluster->lifeCheckNodes[i].stateName, nodeInfo->stateName);
+		gslifeCheckCluster->lifeCheckNodes[i].nodeState = NODE_EMPTY; /* This is local health check state*/
 		gslifeCheckCluster->lifeCheckNodes[i].ID = nodeInfo->id;
 		strcpy(gslifeCheckCluster->lifeCheckNodes[i].hostName, nodeInfo->hostName);
 		strcpy(gslifeCheckCluster->lifeCheckNodes[i].nodeName, nodeInfo->nodeName);
@@ -665,7 +671,7 @@ is_wd_lifecheck_ready(void)
 		/* heartbeat mode */
 		else if (pool_config->wd_lifecheck_method == LIFECHECK_BY_HB)
 		{
-			if (node->ID == 0)	/* local node */
+			if (node->ID == pool_config->pgpool_node_id)	/* local node */
 				continue;
 
 			if (!WD_TIME_ISSET(node->hb_last_recv_time) ||
@@ -682,7 +688,7 @@ is_wd_lifecheck_ready(void)
 		else
 		{
 			ereport(ERROR,
-					(errmsg("checking if watchdog is ready, unkown watchdog mode \"%d\"",
+					(errmsg("checking if watchdog is ready, unknown watchdog mode \"%d\"",
 							pool_config->wd_lifecheck_method)));
 		}
 	}
@@ -886,7 +892,7 @@ check_pgpool_status_by_query(void)
 					continue;
 				node->nodeState = NODE_DEAD;
 				/* It's me! */
-				if (i == 0)
+				if (i == pool_config->pgpool_node_id)
 					inform_node_status(node, "parent process is dead");
 				else
 					inform_node_status(node, "unable to connect to node");
@@ -1008,11 +1014,20 @@ wd_ping_pgpool(LifeCheckNode * node, char* password)
 
 	conn = create_conn(node->hostName, node->pgpoolPort, password);
 	if (conn == NULL)
+	{
+		if(chceck_password_type_is_not_md5(pool_config->wd_lifecheck_user, pool_config->wd_lifecheck_password) == -1)
+		{
+			ereport(ERROR,
+					(errmsg("the password of wd_lifecheck_user %s is invalid format",
+						pool_config->recovery_user),
+					errdetail("wd_lifecheck_password is not allowed to be md5 hashed format")));
+		}
 		return WD_NG;
+	}
 	return ping_pgpool(conn);
 }
 
-/* inner function for issueing lifecheck query */
+/* inner function for issuing lifecheck query */
 static int
 ping_pgpool(PGconn *conn)
 {
@@ -1101,7 +1116,7 @@ wd_ping_all_server(void)
 		WdUpstreamConnectionData *server = (WdUpstreamConnectionData *) lfirst(lc);
 
 		if (server->pid <= 0)
-			server->pid = wd_issue_ping_command(server->hostname, &server->outputfd);
+			server->pid = wd_trusted_server_command(server->hostname);
 
 		if (server->pid > 0)
 			ping_process++;
@@ -1118,15 +1133,19 @@ wd_ping_all_server(void)
 			if (server)
 			{
 				ping_process--;
-				server->reachable = wd_get_ping_result(server->hostname, status, server->outputfd);
+				server->reachable = (status == 0);
 				server->pid = 0;
-				close(server->outputfd);
 				if (server->reachable)
 				{
 					/* one reachable server is all we need */
 					POOL_SETMASK(&UnBlockSig);
 					return true;
 				}
+				if (WIFEXITED(status) == 0 || WEXITSTATUS(status) != 0)
+				{
+					ereport(WARNING,
+							(errmsg("watchdog failed to ping host: \"%s\"", server->hostname)));
+				}
 			}
 			else
 			{
@@ -1140,7 +1159,7 @@ wd_ping_all_server(void)
 				continue;
 			ereport(WARNING,
 					(errmsg("failed to check the ping status of trusted servers"),
-					 errdetail("waitpid failed with reason: %s", strerror(errno))));
+					 errdetail("waitpid failed with reason: %m")));
 			break;
 		}
 	}
